// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "Configuration.h"
#include "Tracer.h"
#include "scanner.h"
#include "Interpreter.h"


TableT
Configuration::defaultSettings(
			unsigned int	numberOfDefaults,
			const Default *	defaults ) const
{
        PROC

	TableT		s;

	for( unsigned int i = 0; i < numberOfDefaults; ++i )
	{
        	s[ defaults[ i ]._name ] = defaults[ i ]._setting;
	} // for

        RETURN s;
}


Configuration::Configuration(
			const string &				fileName,
			const string &				configurationName,
			unsigned int				numberOfDefaults,
			const Default *				defaults )
{
	PROC

	// *****************
	// Set the defaults.
	// *****************
	_settings = defaultSettings( numberOfDefaults, defaults );


	// *********************************
	// Open the specified file, and read
	// the specified configuration.
	// *********************************
	ScannerT	scanner( fileName );
	TableT		lookupTable;
	
	interpreter.addFunctionsToTable( lookupTable );

	while( !scanner.EndOfFile() )
	{
		Token	configuration;

		scanner.match( "configuration" );
		scanner.Match( Token::Identifier, configuration );

		scanner.match( '<' );

		while( !scanner.NextIs( '>' ))
		{
			// ************************************************************
			// The (name, value) pair must appear on a single line.
			// Get the line, and parse it.
			// ************************************************************
			scanner.SetState( ScannerT::LINE );

			// **********************
			// Get (name,value) pair.
			// **********************
			Token		name;
			Token		value;

			scanner.Match( Token::Identifier, name );
			value = scanner.acceptNext();
			scanner.SetState( ScannerT::NORMAL );

			if( !( configuration.asString() == configurationName )) continue;

			_settings[ name.asString() ] = interpreter.EvaluateExpression(
									value,
									lookupTable );

		} // while

		scanner.match( '>' );
	} // while

	ENDPROC
}


const Token &
Configuration::setting( const string & name ) const
{
	PROC
	RETURN _settings.Fetch( name );
}


const TableT &
Configuration::settings() const
{
	PROC
	RETURN _settings;
}

