// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/* Revision History
 *
 * $Id: Interpreter.h,v 1.6 1999/09/21 12:51:23 mthomas Exp $
 *
 */

#ifndef __INTERPRETER_H
#define __INTERPRETER_H


#include "binary_tree.h"
#include "token.h"
#include "table.h"


typedef void * (*CFunctionType)();


struct Lookups
{
	string		name;
	Token		token;

};


class Interpreter
{

	public :

		virtual ~Interpreter() {}

		virtual void addFunctionsToTable( TableT &	table ) const;

		Token EvaluateExpression(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const; /* Of TokenT */

	protected :
		Token EvaluateDefunFunction(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;
		Token EvaluateCFunction(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;
		Token EvaluateInternalFunction(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;

	private :

		Token evaluateBinaryOperation(
				const Token::BTree &	expression,
				TableT &		lookupTable,
				Token (Token::*f)(const Token &) const ) const;

		Token evaluatePlusPlus(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;
		Token evaluateMinusMinus(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;
		Token EvaluateArray(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;
		Token EvaluateIf(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;
		Token EvaluateLoop(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;
		Token EvaluateCompound(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;
		Token EvaluateAddressOf(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;
		Token EvaluateAssignment(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;
		virtual Token EvaluateFunction(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;
		Token EvaluateDefunDeclaration(
				const Token::BTree &	expression,
				TableT &		lookupTable ) const;

};

extern Interpreter interpreter;

void eval( const char * expression, TableT * lookupTablePtr );

TableT Functions(
		struct Lookups *	theseLookups,
		unsigned int		noOfLookups );


#endif

