// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

//
// StreamT.h
//
//
// HISTORY:
//
//    Date        Author   Modifications
//    --------    ------   -------------
//    15/06/97    MT       Initial Version
//
//
//
//
//
//
//
// DESCRIPTION:
//
// Public header file for Stream functions
//
//
// **********************************************************


#ifndef __STREAMT_H
#define __STREAMT_H


#include <istream>
#include <cstdio>

#include "Tracer.h"


class StreamT
{
	public :

				StreamT(
					istream &	thisStream,
					const string &	fileName = "NoName",
					unsigned int	lineNumber = 1,
					unsigned int	columnNumber = 1 );
				virtual ~StreamT(){}
		char		GetNextChar();
		char		peek();
		char		GetNextEscapedChar();
		void		getLine( char * line, streamsize n );
		void		skipLine();
		void		UngetCharacter( char thisCharacter );
		int		EndOfFile() const;
		unsigned int	CurrentLineNo() const;
		unsigned int	CurrentColumnNo() const;
		istream &	stream() { PROC RETURN _is; };
		virtual const string & FileNameOf() const
		{
			PROC
			RETURN _fileName;

		}
		friend ostream & operator<<( ostream &, const StreamT & );
	private :

		istream & 	_is;
		string		_fileName;
		unsigned int	_currentLineNumber;
		unsigned int	_currentColumnNumber;
};


#endif

