// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/* Here we create an extended version of DataType from Dal.h which
   adds signed and unsigned long integers. These are not valid
   container types for the dal, but we need to add them so that
   interpr can define unsigned long arrays which are used to define
   dimensions in certain dal calls, such as addColumn. The test
   harnesses defined the dimensions arrays using Int32 pointers which
   worked on 32-bit systems, but this causes run-time problems in
   64-bit. I'm not sure why the compiler doesn't warn about type
   mismatches - it may be that we are hiding things in void * so it
   has no way of knowing.

   14-MAR-2013 Initial Version (EC) */

#ifndef DAL_TYPES_EXTENDED_H
#define DAL_TYPES_EXTENDED_H

enum interprDataType {
  Bool = 1, /**< Boolean */
  Int8, /**< 8-bit unsigned integers */
  Uint16, /**< 16-bit unsigned integers */
  Int16, /**< 16-bit signed integers */
  Uint32, /**< 32-bit unsigned integers */
  Int32, /**< 32-bit signed integers */
  Real32, /**< 32-bit reals */
  Real64, /**< 64-bit reals */
  DString, /**< Strings of characters */

  // Here are the new types, which may or may not be identical to the
  // int32 types, depending on the architecture

  Ulong,
  Long
};

#endif
