// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/* Revision History
 *
 * $Log: debug.cc,v $
 * Revision 1.4  2002/09/17 09:38:23  gvacanti
 * Committed through make
 *
 * Revision 1.2  2002/06/11 09:18:06  jbakker
 * Committed through make
 *
 * Revision 1.1  1998/09/25 15:54:27  mbeijers
 * Split from package utils.
 *
 * Revision 1.2  1997/09/29 11:37:18  mthomas
 * Reference: Warning Messages
 * Software Changes:
 * 	Modify code so that all Comipler Warnings are removed.
 *
 * Revision 1.1  1997/09/09 13:24:06  mbeijers
 * Moved sources to src subdirectory.
 *
 * Revision 1.1  1997/08/22 13:40:14  mthomas
 * Reference: First Check-in
 * Software Change:
 * 	First stable prototype.
 *
 *
 * $Id: debug.cc,v 1.4 2002/09/17 09:38:23 gvacanti Exp $
 *
 */

// #ifndef lint
// static char version_id[] = "@@(#)$Id: debug.cc,v 1.4 2002/09/17 09:38:23 gvacanti Exp $";
// #endif

//
//	File:	debug.cc
//
//	Title:	Implementation file for debug facility
//
//	Author:	M Thomas
//
//	Date:	13/06/97
//
//


#include <iostream>
#include <cstdarg>
#include <cstdlib>

#include "debug.h"



// static unsigned int NoOfNames = 0;

static struct NamesT
	{
		char *		Name;
		NamesT *	Next;
	} * Names = (NamesT *)0;

static unsigned int No = 0;


static void
DebugIndent( void )
{
	static const unsigned int	SpaceLength = 4096;
	static char			Space[ SpaceLength ];
	static unsigned int		FirstTime = 1;

	if( FirstTime )
	{
		for( unsigned int i = 0; i < SpaceLength - 1; i++ )
		{
			Space[ i ] = ' ';
		} // for

		Space[ SpaceLength - 1 ] = '\0';
		FirstTime = 0;
	} // if


	cout << Space + SpaceLength - 1 - No * 2;
	//printf( "%s", Space + SpaceLength - 1 - No * 2 );


} /* static void DebugIndent () */


static void
DebugAdd( char		* Name )
{
	NamesT		* New;

	New = new NamesT;
	New -> Name = Name;
	New -> Next = Names;
	Names = New;

	No++;


} /* static void DebugAdd () */


static void
DebugRemove( void )
{
	NamesT		* New;


	New = Names -> Next;

	delete Names;
	Names = New;

	No--;

} /* static void DebugRemove () */


void
DebugEnterFunction(
	char		* FileName,
	char		* Name )
{

	DebugIndent();

	cout << FileName << ":" << Name << ": ENTER" << endl;

	//printf( "%s:%s: ENTER\n", FileName, Name );

	DebugAdd( Name );

} /* void DebugEnterFunction () */


void
DebugExitFunction(
	char		* FileName )
{
	char	* Current;

	Current = Names -> Name;
	DebugRemove();
	DebugIndent ();

	cout << FileName << ":" << Current << ": EXIT" << endl;

	//printf ( "%s:%s: EXIT\n", FileName, Current );

} /* void DebugExitFunction () */


void
DebugPrintf (
	char		* FileName,
	char		* Format,
	...		/* Arguments */)
{
	static char	PrintString[ 1024 ];
	va_list		Ptr;

	va_start (Ptr, Format);

	DebugIndent();

	cout << FileName << ":" << Names -> Name << ": ";

	//printf( "%s:%s: ", FileName, Names -> Name );

	vsprintf( PrintString, Format, Ptr);

	cout << PrintString << endl;

	//printf( "\n" );

	va_end (Ptr);

} /* void DebugPrintf () */


