// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.


/* Revision History
 *
 * $Id: evalperl.cc,v 1.9 2002/09/17 09:38:23 gvacanti Exp $
 *
 */

#include <ostream>
#include <cerrno>

extern "C" {
#include "EXTERN.h"
#include <cassert>
//#include <cstring>
#include <perl.h>
}


#include "evalperl.h"


/*perl_call ("foo",
       "s",    "hello",
       "i",    2,
       "d",    5.4,
       "OUT",
       "i",    &i,
       "s",    buf,
       NULL);
*/
typedef struct {
    char type;       
    void *pdata;
} Out_Param;


int
perl_call_va( const char *subname, ... )
{

	va_list vl;

	va_start (vl, subname);
 
	perl_call_vl( subname, vl );

   	va_end(vl);

	return 1;
}


int
//perl_call_va (const char *subname, ...)
perl_call_vl( const char *subname, va_list vl )
{
	char *p;
	char *str = NULL; int i = 0; double d = 0;
	int  nret = 0; /* number of return params expected*/
	int ii=0;
	void * pp;
	Out_Param op[20];
	int out = 0;
	int result = 0;

    	djSP;
    	ENTER;
    	SAVETMPS;
    	PUSHMARK(sp);
 
    	while ((p = va_arg(vl, char *)))
	{
        	switch (*p)
        	{
        		case 's' :
            			if (out)
				{
                			op[nret].pdata = (void*) va_arg(vl, char *);
                			op[nret++].type = 's';
            			}
				else
				{
                			str = va_arg(vl, char *);
         				ii = strlen(str);
                			XPUSHs(sv_2mortal(newSVpv(str,ii)));
            			} // if
            			break;
        		case 'i' :
            			if (out)
				{
			                op[nret].pdata = (void*) va_arg(vl, int *);
                			op[nret++].type = 'i';
            			}
				else
				{
                			ii = va_arg(vl, int);
                			XPUSHs(sv_2mortal(newSViv(ii)));
            			} // if
            			break;
			case 'p' :
				if( out )
				{
					// To be Implemented.
				}
				else
				{
					pp = va_arg(vl, void *);
                			XPUSHs(sv_2mortal(newSViv((IV)pp)));
				} // if
				break;
        		case 'd' :
            			if (out)
				{
                			op[nret].pdata = (void*) va_arg(vl, double *);
                			op[nret++].type = 'd';
            			}
				else
				{
               				d = va_arg(vl, double);
               				XPUSHs(sv_2mortal(newSVnv(d)));
            			} // if
            			break;
        		case 'O':
            			out = 1;  /* Out parameters starting */
            			break;
        		default:
             			fprintf (stderr, "perl_eval_va: Unknown option \'%c\'.\n"
                               "Did you forget a trailing NULL ?\n", *p);
            			return 0;
        	} // switch
	} // while
   
    	PUTBACK;
    	result = perl_call_pv((char*)subname, (nret == 0) ? G_DISCARD :
                                   (nret == 1) ? G_SCALAR  :
                                                 G_ARRAY  );
 
    
 
    	SPAGAIN;
    	if (nret > result)
	{
        	nret = result;
	} // if
 
    	for (i = --nret; i >= 0; i--)
	{
        	switch (op[i].type)
		{
		        case 's':
				str = POPp;
				strcpy((char *)op[i].pdata, str);
				break;
			case 'i':
				ii = POPi;
				*((int *)(op[i].pdata)) = ii;
				break;
			case 'd':
				d = POPn;
				*((double *) (op[i].pdata)) = d;
				break;
			case 'p' :
				// To be Implemented.
				break;
		} // switch
	} // for
   
	FREETMPS ;
	LEAVE ;

	return result;
}
 

int
perl_get_int( const char * variableName, int * pvalue )
{
	SV * sv = perl_get_sv( (char *)variableName, FALSE );
	*pvalue = SvIV( sv );
	return 0;
}

