// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/* Revision History
 *
 * $Id: perlutils.cc,v 1.2 2009/07/06 14:20:08 aibarra Exp $
 *
 */

#include "perlutils.h"

/*
#include <cassert>
#include <cstring>
#include <std::cerrno>
*/
#include <cstring>
#include <cstdio>

// Under some conditions, the Perl headers are not protected against
// name mangling. Make sure they are.
extern "C" {
  //#define _INTTYPES_H	1   // nice for GCC-3.1
#include "EXTERN.h"
#include "perl.h"
}


typedef struct {
    char type;
    void *pdata;
} Out_Param;


int
perl_eval_va (const char *str, ...)
{
    /* Evals a string, returns -1 if unsuccessful, else returns
     *  the number of return params
     *  char buf[10]; int a;
     *  perl_eval_va ("$a = 10; ($a, $a+1)",
     *                "i", &a,
     *                "s", buf,
     *                 NULL);
     */

    SV*       sv     = newSVpv((char *)str,0);
    va_list   vl;
    char      *p     = NULL;
    int       i      = 0;
    int       nret   = 0;     /* number of return params expected*/
    int       result = 0;
    Out_Param op[20];
    int ii; double d;

    djSP; /* GV -- this used to be dSP, but that fails on Linux because of a problem with errno and threads */
    ENTER;
    SAVETMPS;
    PUSHMARK(sp);
    va_start (vl, str);

    while ((p = va_arg(vl, char *))) {
        if ((*p != 's') && (*p != 'i') && (*p != 'd')) {
            fprintf (stderr, "perl_eval_va: Unknown option \'%c\'.\n"
                              "Did you forget a trailing NULL ?\n", *p);
            return -1;
        }
        op[nret].pdata = (void*) va_arg(vl, char *);
        op[nret++].type = *p;
    }
    va_end(vl);
    PUTBACK;
    result = perl_eval_sv(sv, (nret == 0) ? G_DISCARD :
                              (nret == 1) ? G_SCALAR  :
                                            G_ARRAY  );

    SPAGAIN;
    if (SvTRUE(ERRSV)) { /* ERRSV == $@ */
        fprintf (stderr, "Eval error: %s", SvPV(ERRSV, PL_na)) ;
        return -1;
    }
    SvREFCNT_dec(sv);
    /*printf ("nret: %d, result: %d\n", nret, result);*/
    if (nret > result)
        nret = result;

    for (i = --nret; i >= 0; i--) {
        switch (op[i].type) {
        case 's':
            str = POPp;
            /*printf ("String: %s\n", str);*/
            strcpy((char *)op[i].pdata, str);
            break;
        case 'i':
            ii = POPi;
            /*printf ("Int: %d\n", ii);*/
            *((int *)(op[i].pdata)) = ii;
            break;
        case 'd':
            d = POPn;
            /*printf ("Double: %f\n", d);*/
            *((double *) (op[i].pdata)) = d;
            break;
        }
   }
   FREETMPS ;
   LEAVE;
   return result;
}



// #ifdef __cplusplus
// extern "C" {
// #endif

// #include <EXTERN.h>
// #include <perl.h>

// #ifdef __cplusplus
// }
// #  ifndef EXTERN_C
// #    define EXTERN_C extern "C"
// #  endif
// #else
// #  ifndef EXTERN_C
// #    define EXTERN_C extern
// #  endif
// #endif

#ifndef EXTERN_C
#  define EXTERN_C extern "C"
#endif

EXTERN_C void xs_init _((void));

EXTERN_C void boot_DynaLoader _((CV* cv));

EXTERN_C void
xs_init()
{
	/* AI gcc4 porting
	 * deprecated conversion from string constant to 'char*' => cast (char*)
	 */
	char *file = (char*)__FILE__;
	dXSUB_SYS;

	/* DynaLoader is a special case */
	newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
}

static char* initialisePerlName=0;
void
initialisePerl( const char* perllib )

{
  if (initialisePerlName) delete [] initialisePerlName;
  initialisePerlName=new char[strlen(perllib)+1];
  strcpy(initialisePerlName,perllib);

  /* AI gcc4 porting
   * deprecated conversion from string constant to 'char*' => cast (char*)
   */
  static char*		  argv[] = { (char*)"", initialisePerlName };
  static PerlInterpreter* perlInterpreter;

  perlInterpreter = perl_alloc();
  perl_construct( perlInterpreter );
  perl_parse( perlInterpreter, xs_init, 2, argv, 0 );

}


int
evalperl( const char * expression )
{

	int				status;

	status = perl_eval_va( (char *)expression, 0 );

	return status;

}


