// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/* Revision History
 *
 * $Id: table.h,v 1.6 2002/09/17 09:38:24 gvacanti Exp $
 *
 */


/**************************************************************************/
/*                                                                        */
/*  FILENAME                                                              */
/*  ========                                                              */
/*  table.h                                                               */
/*                                                                        */
/*                                                                        */
/*  SUBSYSTEM                                                             */
/*  =========                                                             */
/*  XMM SAS - UTILITIES                                                   */
/*                                                                        */
/*                                                                        */
/*  TITLE                                                                 */
/*  =====                                                                 */
/*  Public header file for interface to TableT type.                      */
/*                                                                        */
/*                                                                        */
/*  AUTHORS                                                               */
/*  =======                                                               */
/*  Mark Thomas                                                           */
/*  CARA Software & Services Ltd                                          */
/*                                                                        */
/*                                                                        */
/*  CONSTRAINTS                                                           */
/*  ===========                                                           */
/*  All typedefs and functions given in this header file are for          */
/*  public use.                                                           */
/*                                                                        */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*  ===========                                                           */
/*  Public Header file for interface to TableT type.                      */
/*                                                                        */
/*                                                                        */
/*  EXAMPLE                                                               */
/*  =======                                                               */
/*                                                                        */
/*                                                                        */
/*  #include "table.h"                                                    */
/*                                                                        */
/*                                                                        */
/*  main()                                                                */
/*  {                                                                     */
/*  	TableT		Table;                                            */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/*  }                                                                     */
/*                                                                        */
/*                                                                        */
/**************************************************************************/



#ifndef __TABLE_H
#define __TABLE_H 

#include "token.h"

#include <iostream>
#include <list>



class TableT
{

	public :

		TableT();
		TableT( const TableT & );
		TableT & operator=( const TableT & Table );


		~TableT();


		Token *	Insert(
				const string &	itemName,
				const Token &	thisItem );


		Token & Fetch(
				const string &		itemName ) const;

		Token & operator[] ( const string &	itemName );

		int IsFound(
				const string &		itemName ) const;

		void first();
		Token & value() const;
		void modify(
			const string &	itemName,
			const Token &	withThisValue );
		const char * name() const;
		void next();
		int more();
		void append( const TableT & thisTable );

		friend ostream& operator<<(

				ostream &	toThisStream,
				const TableT &	thisTable );

	private :

		struct TableItem
		{
			string	_name;
			Token	_value;
		};

		list<TableItem>			_table;
		list<TableItem>::iterator	_current;
};


#endif

